const config = require('config')
const { get, chain } = require('lodash')

const { services } = require('pubsweet-component-helper-service')
const { getEmailCopy } = require('./emailCopy')

const unsubscribeSlug = config.get('unsubscribe.url')

module.exports = {
  sendReviewersEmail: async ({ email, baseUrl, reviewers }) => {
    reviewers.forEach(reviewer => {
      email.toUser = {
        email: reviewer.email,
        name: `${reviewer.lastName}`,
      }
      email.content.unsubscribeLink = services.createUrl(
        baseUrl,
        unsubscribeSlug,
        {
          id: reviewer.id,
        },
      )
      const { html, text } = email.getBody({
        body: { paragraph: reviewer.paragraph },
      })
      email.sendEmail({ html, text })
    })
  },
  getSubmittedReviewers: async ({
    UserModel,
    titleText,
    fragmentHelper,
    recommendation,
  }) => {
    const emailType =
      recommendation === 'publish'
        ? 'submitted-reviewers-after-publish'
        : 'submitted-reviewers-after-reject'

    const reviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'submitted',
    })).map(rev => ({
      ...rev,
      emailType,
      ...getEmailCopy({
        emailType,
        titleText,
      }),
    }))

    return reviewers
  },
  getNoResponseReviewers: async ({ fragmentHelper, UserModel, titleText }) => {
    const acceptedReviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'accepted',
    })).map(rev => ({
      ...rev,
      ...getEmailCopy({
        emailType: 'accepted-reviewers-after-recommendation',
        titleText,
      }),
    }))

    const pendingReviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'pending',
    })).map(rev => ({
      ...rev,
      ...getEmailCopy({
        emailType: 'pending-reviewers-after-recommendation',
        titleText,
      }),
    }))

    const reviewers = [...acceptedReviewers, ...pendingReviewers]
    return reviewers
  },
  updateEmailContentForReviewers: ({
    email,
    subject,
    customId,
    signatureName,
  }) => {
    email.content.subject = `${customId}: ${subject}`
    email.content.signatureName = signatureName
    return email
  },
  sendHandlingEditorEmail: ({
    email,
    comments,
    emailType,
    titleText,
    targetUserName,
  }) => {
    const { html, text } = email.getBody({
      body: getEmailCopy({
        emailType,
        titleText,
        comments,
        targetUserName,
      }),
    })
    email.sendEmail({ html, text })
  },
  updateEmailContentForHE: ({
    email,
    baseUrl,
    eicName,
    customId,
    heLastName,
    handlingEditor,
    recommendationType,
  }) => {
    if (recommendationType === 'review') {
      email.content.subject = `${customId}: A review has been submitted`
    } else {
      email.content.subject = `${customId}: Editorial decision confirmed`
    }

    email.toUser = {
      email: handlingEditor.email,
      name: heLastName,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: handlingEditor.id,
      },
    )

    email.content.signatureName = eicName
    delete email.content.signatureJournal

    return email
  },
  getEmailTypeByRecommendationForHE: ({
    recommendation,
    recommendationType,
  }) => {
    let emailType
    if (recommendationType === 'review') {
      emailType = 'he-review-submitted'
    } else {
      switch (recommendation) {
        case 'return-to-handling-editor':
          emailType = 'he-manuscript-return-with-comments'
          break
        case 'publish':
          emailType = 'he-manuscript-published'
          break
        case 'reject':
          emailType = 'he-manuscript-rejected'
          break
        default:
          throw new Error(`undefined recommendation: ${recommendation} `)
      }
    }

    return emailType
  },
  getEiCCommentsForHE: ({ newRecommendation }) => {
    const eicComments = chain(newRecommendation)
      .get('comments')
      .find(comm => !comm.public)
      .get('content')
      .value()

    return eicComments
  },
  sendEiCsEmail: async ({
    email,
    customId,
    titleText,
    userHelper,
    targetUserName,
    recommendation: { recommendation, comments: recComments = [] },
  }) => {
    let emailType

    switch (recommendation) {
      case 'minor':
      case 'major':
        emailType = 'eic-request-revision-from-he'
        email.content.subject = `${customId}: Revision requested`
        break
      case 'publish':
        emailType = 'eic-recommend-to-publish-from-he'
        email.content.subject = `${customId}: Recommendation to publish`
        break
      case 'reject':
        emailType = 'eic-recommend-to-reject-from-he'
        email.content.subject = `${customId}: Recommendation to reject`
        break
      default:
        throw new Error(`undefined recommendation: ${recommendation} `)
    }
    delete email.content.signatureJournal

    const privateNote = recComments.find(comm => !comm.public)
    const content = get(privateNote, 'content')
    const comments = content
      ? `The editor provided the following comments: "${content}"`
      : ''

    const editors = (await userHelper.getEditorsInChief()).map(eic => ({
      ...eic,
      ...getEmailCopy({
        comments,
        emailType,
        titleText,
        targetUserName,
      }),
    }))

    editors.forEach(eic => {
      email.toUser = {
        email: eic.email,
        name: `${eic.firstName} ${eic.lastName}`,
      }
      const { html, text } = email.getBody({
        body: { paragraph: eic.paragraph },
      })
      email.sendEmail({ html, text })
    })
  },
  sendAuthorsEmail: async ({ email, authors, baseUrl }) => {
    authors.forEach(author => {
      email.toUser = {
        email: author.email,
        name: `${author.lastName}`,
      }
      email.content.unsubscribeLink = services.createUrl(
        baseUrl,
        unsubscribeSlug,
        {
          id: author.id,
        },
      )
      const { html, text } = email.getBody({
        body: {
          paragraph: author.paragraph,
        },
      })
      email.sendEmail({ html, text })
    })
  },
  getSubmittingAuthor: ({
    title,
    journalName,
    authorNoteText,
    submittingAuthor,
  }) => {
    const author = {
      ...submittingAuthor,
      ...getEmailCopy({
        emailType: 'author-request-to-revision',
        titleText: `your submission "${title}" to ${journalName}`,
        comments: authorNoteText,
      }),
    }

    return author
  },
  getPrivateNoteTextForAuthor: ({ newRecommendation }) => {
    const authorNote = newRecommendation.comments.find(comm => comm.public)
    const content = get(authorNote, 'content')
    const authorNoteText = content ? `Reason & Details: "${content}"` : ''

    return authorNoteText
  },
  updateEmailContentForSA: ({ email, customId, signatureName }) => {
    email.content.subject = `${customId}: Revision requested`
    email.content.signatureName = signatureName
  },
  getHEComments: ({ heRecommendation }) => {
    const heComments = get(heRecommendation, 'comments', [])
    if (heComments.length === 0) return

    const publicComment = heComments.find(comm => comm.public)
    const content = get(publicComment, 'content')
    if (!content) {
      throw new Error('a public comment cannot be without content')
    }
    const comments = `Please find our editorial comments below<br/>: "${content}"`

    return comments
  },
  getAllAuthors: ({ title, comments, emailType, fragmentAuthors }) => {
    const authors = fragmentAuthors.map(author => ({
      ...author,
      ...getEmailCopy({
        comments,
        emailType,
        titleText: `your manuscript titled "${title}"`,
      }),
    }))

    return authors
  },
  sendSubmittingAuthorEmail: ({ email, author, baseUrl }) => {
    email.toUser = {
      email: author.email,
      name: `${author.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: author.id,
      },
    )
    const { html, text } = email.getBody({
      body: {
        paragraph: author.paragraph,
      },
    })
    email.sendEmail({ html, text })
  },
  updateEmailContentForAllAuthors: ({ email, recommendation, customId }) => {
    switch (recommendation) {
      case 'publish':
        email.content.subject = `${customId}: Manuscript accepted`
        break
      case 'reject':
        email.content.subject = `${customId}: Manuscript rejected`
        break
      default:
        throw new Error(`Undefined recommendation: ${recommendation}`)
    }

    return email
  },
  getEmailTypeByRecommendationForAuthors: ({ recommendation }) => {
    let emailType
    switch (recommendation) {
      case 'publish':
        emailType = 'author-manuscript-published'
        break
      case 'reject':
        emailType = 'author-manuscript-rejected'
        break
      default:
        throw new Error(`Undefined recommendation: ${recommendation}`)
    }

    return emailType
  },
}
