const config = require('config')
const { chain, get, isEmpty } = require('lodash')

const {
  User,
  Email,
  services,
  Fragment,
  Collection,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')
const helpers = require('./helpers')

const editorialAssistantEmail = config.get('mailer.editorialAssistant')
const unsubscribeSlug = config.get('unsubscribe.url')
const journalName = config.get('journal.name')

module.exports = {
  async sendNotifications({
    hasEQA,
    baseUrl,
    fragment,
    UserModel,
    collection,
    targetUserName,
    isEditorInChief,
    newRecommendation,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const fragmentAuthors = await fragmentHelper.getAuthorData({ UserModel })

    const subjectBaseText = `${collection.customId}: Manuscript`
    const titleText = `the manuscript titled "${parsedFragment.title}" by ${
      fragmentAuthors.submittingAuthor.firstName
    } ${fragmentAuthors.submittingAuthor.lastName}`

    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()

    let email = new Email({
      type: 'user',
      content: {
        signatureName: eicName,
        unsubscribeLink: baseUrl,
        ctaText: 'MANUSCRIPT DETAILS',
        signatureJournal: journalName,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
      },
    })

    const { recommendation, recommendationType } = newRecommendation

    // the EiC recommends to publish so an email to the EQA needs to be sent,
    // one requesting approval or one informing them that the manuscript has been published
    if (isEditorInChief && recommendation === 'publish') {
      sendEQAEmail({
        email,
        eicName,
        baseUrl,
        titleText,
        collection,
        subjectBaseText,
      })
    }

    const hasPeerReview = (collection = {}) =>
      !isEmpty(collection.handlingEditor)
    const { customId } = collection
    const collHelper = new Collection({ collection })

    if (
      (isEditorInChief || recommendationType === 'review') &&
      hasPeerReview(collection)
    ) {
      if (recommendation !== 'publish' || hasEQA) {
        // the request came from either the Editor in Chief or a reviewer, so the HE needs to be notified
        email = helpers.updateEmailContentForHE({
          email,
          baseUrl,
          eicName,
          customId,
          recommendationType,
          heLastName: collHelper.getHELastName(),
          handlingEditor: get(collection, 'handlingEditor', {}),
        })
        const emailType = helpers.getEmailTypeByRecommendationForHE({
          recommendation,
          recommendationType,
        })
        const comments = helpers.getEiCCommentsForHE({ newRecommendation })
        helpers.sendHandlingEditorEmail({
          email,
          comments,
          emailType,
          titleText,
          targetUserName,
        })
      }
    }

    if (
      recommendationType !== 'review' &&
      recommendation !== 'return-to-handling-editor'
    ) {
      if (isEditorInChief) {
        if (recommendation !== 'publish' || hasEQA) {
          // send all authors email
          const emailType = helpers.getEmailTypeByRecommendationForAuthors({
            recommendation,
          })
          const comments = helpers.getHEComments({
            heRecommendation: parsedFragment.heRecommendation,
          })
          const authors = helpers.getAllAuthors({
            comments,
            emailType,
            title: parsedFragment.title,
            fragmentAuthors: fragmentAuthors.activeAuthors,
          })
          email = helpers.updateEmailContentForAllAuthors({
            email,
            customId,
            recommendation,
          })
          helpers.sendAuthorsEmail({ email, authors, baseUrl })
        }
      }

      if (collection.status === 'revisionRequested') {
        // send SA email
        const authorNoteText = helpers.getPrivateNoteTextForAuthor({
          newRecommendation,
        })
        const author = helpers.getSubmittingAuthor({
          authorNoteText,
          journalName,
          title: parsedFragment.title,
          submittingAuthor: fragmentAuthors.submittingAuthor,
        })
        email = helpers.updateEmailContentForSA({
          email,
          customId,
          signatureName: get(collection, 'handlingEditor.name', eicName),
        })
        helpers.sendSubmittingAuthorEmail({ email, author, baseUrl })
      }

      if (hasPeerReview(collection)) {
        let reviewers
        if (isEditorInChief) {
          if (recommendation !== 'publish' || hasEQA) {
            email = helpers.updateEmailContentForReviewers({
              email,
              customId,
              signatureName: eicName,
              subject: `A manuscript you reviewed has been rejected`,
            })
            reviewers = helpers.getSubmittedReviewers({
              UserModel,
              titleText,
              fragmentHelper,
              recommendation,
            })
          }
        } else {
          email = helpers.updateEmailContentForReviewers({
            email,
            customId,
            subject: `Review no longer required`,
            signatureName: get(collection, 'handlingEditor.name', eicName),
          })

          reviewers = helpers.getNoResponseReviewers({
            UserModel,
            titleText,
            fragmentHelper,
          })
        }
        helpers.sendReviewersEmail({ email, baseUrl, reviewers })

        if (!isEditorInChief) {
          helpers.sendEiCsEmail({
            email,
            baseUrl,
            customId,
            titleText,
            userHelper,
            recommendation: newRecommendation,
            targetUserName: collHelper.getHELastName(),
          })
        }
      }
    }
  },
}

const sendEQAEmail = ({
  email,
  eicName,
  baseUrl,
  titleText,
  collection,
  subjectBaseText,
}) => {
  let emailType
  switch (collection.status) {
    case 'accepted':
      emailType = 'eqa-manuscript-published'
      email.content.subject = `${subjectBaseText} Decision`
      break
    case 'inQA':
      emailType = 'eqa-manuscript-request-for-approval'
      email.content.subject = `${subjectBaseText} Request for EQA Approval`
      email.content.ctaLink = services.createUrl(
        baseUrl,
        config.get('eqa-decision.url'),
        {
          collectionId: collection.id,
          customId: collection.customId,
          token: collection.technicalChecks.token,
        },
      )
      email.content.ctaText = 'MAKE DECISION'
      break
    default:
      throw new Error(
        `Cannot send EQA email when collection status is ${collection.status} `,
      )
  }

  email.toUser = {
    email: editorialAssistantEmail,
    name: 'Editorial Assistant',
  }

  email.content.unsubscribeLink = baseUrl
  email.content.signatureName = eicName

  const { html, text } = email.getBody({
    body: getEmailCopy({
      eicName,
      titleText,
      emailType,
      customId: collection.customId,
    }),
  })
  email.sendEmail({ html, text })
}
